@@ -4,7 +4,7 @@
 #
 # copyright (c) 2002-8, Hao Wu and Karl W. Broman
 # 
-# Last modified Feb, 2008
+# Last modified Jul, 2008
 # first written Jul, 2002
 #
 # Modified by Hao Wu Feb 2005 for the following:
@@ -27,6 +27,11 @@ function (cross, pheno.col = 1, mname1, mark1, geno1, mname2,
   if(!sum(class(cross) == "cross")) 
     stop("The first input variable must be an object of class cross")
 
+  if(LikePheVector(pheno.col, nind(cross), nphe(cross))) {
+    cross$pheno <- cbind(pheno.col, cross$pheno)
+    pheno.col <- 1
+  }
+
   if(length(pheno.col) > 1) {
     pheno.col <- pheno.col[1]
     warning("effectplot can take just one phenotype; only the first will be used")
@@ -100,7 +105,10 @@ function (cross, pheno.col = 1, mname1, mark1, geno1, mname2,
         mname2 <- "Marker 2"
     }
   }
-  else mark2 <- NULL
+  else {
+    mark2 <- NULL
+    geno2 <- NULL
+  }
   ### till now, mark1 and mark2 are genotype data in matrix
   
   ########################################################
@@ -173,14 +181,6 @@ function (cross, pheno.col = 1, mname1, mark1, geno1, mname2,
     }
     else { # otherwise, generate a standard one
       geno1 <- getgenonames(type, "A", cross.attr=attributes(cross))
-#      if(type == "bc") 
-#        geno1 <- c("AA", "AB")
-#      else if(type == "f2")
-#        geno1 <- c("AA", "AB", "BB")
-#      else if(type == "riself" || type == "risib") 
-#        geno1 <- c("AA", "BB")
-#      else if(type == "4way") 
-#        geno1 <- c("AC", "BC", "AD", "BD")
       if(length(levels(tmpf)) > length(geno1)) 
         geno1 <- c(geno1, rep("?", length(levels(tmpf)) - 
                               length(geno1)))
@@ -213,14 +213,6 @@ function (cross, pheno.col = 1, mname1, mark1, geno1, mname2,
       }
       else { # otherwise, generate a standard one
         geno2 <- getgenonames(type, "A", cross.attr=attributes(cross))
-#        if(type == "bc") 
-#          geno2 <- c("AA", "AB")
-#        else if(type == "f2")
-#          geno2 <- c("AA", "AB", "BB")
-#        else if(type == "riself" || type == "risib") 
-#          geno2 <- c("AA", "BB")
-#        else if(type == "4way") 
-#          geno2 <- c("AC", "BC", "AD", "BD")
         if(length(levels(tmpf)) > length(geno2)) 
            geno2 <- c(geno2, rep("?", length(levels(tmpf)) - 
                                  length(geno2)))
@@ -241,9 +233,10 @@ function (cross, pheno.col = 1, mname1, mark1, geno1, mname2,
   # the output will be a data frame. For two-marker case,
   # the rows corresponding to the first marker and the columns
   # corresponding to the second marker
-  result <- effectplot.calmeanse(pheno, mark1, mark2, ndraws, var.flag)
+  result <- effectplot.calmeanse(pheno, mark1, mark2, geno1, geno2, ndraws, var.flag)
   means <- result$Means
   ses <- result$SEs
+
   
   # assign column and row names
   if(is.null(mark2)) {
@@ -278,7 +271,6 @@ function (cross, pheno.col = 1, mname1, mark1, geno1, mname2,
     colnames(result$SEs) <- paste(mname2, geno2, sep = ".")
   }
 
-  
   # calculate lo's and hi's for plot
   lo <- means - ses
   hi <- means + ses
@@ -320,14 +312,14 @@ function (cross, pheno.col = 1, mname1, mark1, geno1, mname2,
     
     # x axis limits
     if(is.null(mark2)) { # one marker
-      u <- sort(unique(as.vector(mark1)))
+      u <- seq(along=geno1)
       d <- diff(u[1:2])
-      xlimits <- c(min(mark1) - d/4, max(mark1) + d/4)
+      xlimits <- c(min(u) - d/4, max(u) + d/4)
     }
     else { # two markers
-      u <- sort(unique(as.vector(mark2)))
+      u <- seq(along=geno2)
       d <- diff(u[1:2])
-      xlimits <- c(min(mark2) - d/4, max(mark2) + d/4)
+      xlimits <- c(min(u) - d/4, max(u) + d/4)
     }
 
     ## fix of x limits
@@ -431,7 +423,7 @@ function (cross, mname)
   # determine marker type - it could be a marker, a pseudomarker or a phenotype
   mar.type <- "none"
   # regular expression pattern for a pseudomarker names
-  pm.pattern <- "c[0-9,X]*\\.loc.*" # pseudomarker names will be like "c1.loc10"
+  pm.pattern <- "^c.*\\.loc.*$" # pseudomarker names will be like "c1.loc10"
   if(mname %in% names(cross$pheno)) { # this is a phenotype
     mar.type <- "pheno"
     idx.pos <- which(mname==names(cross$pheno))
@@ -439,13 +431,13 @@ function (cross, mname)
   else if(length(grep(pm.pattern, mname)) > 0) { # like "c1.loc10", this is a pseudomarker
     # note that the column names for draws is like "loc10",
     # so I need to take the part after "." in mname
-    tmp <- unlist(strsplit(mname, "\\."))
-    chr <- substr(tmp[1],2,nchar(tmp[1])) # this will be like 1 or "X"
+    tmp <- unlist(strsplit(mname, "loc"))
+    chr <- substr(tmp[1],2,nchar(tmp[1])-1) # this will be like 1 or "X"
     if( !(chr %in% names(cross$geno)) )
       stop("Couldn't find marker ", mname)
     mar.type <- "pm"
     chrtype <- class(cross$geno[[chr]])
-    pm.name <- paste(tmp[-1],collapse=".") # this will be like loc10
+    pm.name <- paste("loc", tmp[2],sep="") # this will be like loc10
     idx.pos <- which(pm.name==colnames(cross$geno[[chr]]$draws))
     if(length(idx.pos) == 0)
       stop("Couldn't find marker ", mname)
@@ -528,7 +520,9 @@ function (cross, mname)
 # if ndraws > 1 (has pseudomarker),
 # loop thru the draws
 ##############################################
-effectplot.calmeanse <- function(pheno, mark1, mark2, ndraws, var.flag=c("pooled","group")) {
+effectplot.calmeanse <-
+function(pheno, mark1, mark2, geno1, geno2, ndraws, var.flag=c("pooled","group"))
+{
   # local variables
   nind <- length(pheno) 
   # method to calculate variances for estimated QTL effects
@@ -539,18 +533,18 @@ effectplot.calmeanse <- function(pheno, mark1, mark2, ndraws, var.flag=c("pooled
   
   if(is.null(mark2)) { # if mark2 is missing
     if(ndraws > 1) { # more than one draws
-      mark1.level <- levels(as.factor(mark1)) # level for mark1
+      mark1.level <- seq(along=geno1) # level for mark1
       # init 
       means.all <- matrix(NA, nrow=ndraws, ncol=length(mark1.level))
-      colnames(means.all) <- sort(unique(as.vector(mark1)))
+      colnames(means.all) <- mark1.level
       vars.all <- matrix(NA, nrow=ndraws, ncol=length(mark1.level))
-      colnames(vars.all) <- sort(unique(as.vector(mark1)))
+      colnames(vars.all) <- mark1.level
       weight <- rep(0, ndraws) # weight for draws
       # loop thru draws
       for(i in 1:ndraws) {
         mark1.tmp <- mark1[,i] # data for current draw
         # fit a regression - this is used to calculate the weights
-        mark1.factor <- as.factor(mark1.tmp)
+        mark1.factor <- factor(mark1.tmp, mark1.level)
         lm.tmp <- lm(pheno~mark1.factor-1)
         rss <- sum(lm.tmp$residuals^2)
         # compute the weight
@@ -578,12 +572,20 @@ effectplot.calmeanse <- function(pheno, mark1, mark2, ndraws, var.flag=c("pooled
       ses <- sqrt(meanvar+varmean)
     }
     else { # ndraws is 1
+      u <- sort(unique(mark1))
+      if(any(!(u %in% seq(along=geno1)))) {
+        newmark1 <- mark1
+        for(i in seq(along=u)) 
+          newmark1[mark1==u[i]] <- i
+        mark1 <- newmark1
+      }
       means <- tapply(pheno, mark1, mean, na.rm = TRUE)
+
       if(var.flag == "group") { # use group variance
         ses <- tapply(pheno, mark1, function(a) sd(a, na.rm = TRUE)/sqrt(sum(!is.na(a))))
       }
       else { # use pooled variance
-        mark1.factor <- as.factor(mark1)
+        mark1.factor <- factor(mark1, seq(along=geno1))
         lm.tmp <- lm(pheno~mark1.factor-1)
         rss <- sum(lm.tmp$residuals^2)
         ses <- tapply(mark1, mark1, function(a) sqrt(rss/nind/length(a)))
@@ -593,21 +595,40 @@ effectplot.calmeanse <- function(pheno, mark1, mark2, ndraws, var.flag=c("pooled
   
   else { # with mark2
     if(ndraws > 1) {
-      mark1.level <- levels(as.factor(mark1)) # level for mark1
-      mark2.level <- levels(as.factor(mark2)) # level for mark2
+      mark1.level <- seq(along=geno1) # level for mark1
+      mark2.level <- seq(along=geno2) # level for mark2
+
+      u <- sort(unique(as.numeric(mark1)))
+      if(any(!(u %in% seq(along=geno1)))) {
+        newmark1 <- mark1
+        for(i in seq(along=u)) 
+          for(j in 1:ncol(mark2)) 
+            newmark1[mark1[,j]==u[i],j] <- i
+        mark1 <- newmark1
+      }
+
+      u <- sort(unique(as.numeric(mark2)))
+      if(any(!(u %in% seq(along=geno2)))) {
+        newmark2 <- mark2
+        for(i in seq(along=u)) 
+          for(j in 1:ncol(mark2)) 
+            newmark2[mark2[,j]==u[i],j] <- i
+        mark2 <- newmark2
+      }
+
       # init 
       means.all <- array(NA, c(length(mark1.level), length(mark2.level), ndraws))
-      dimnames(means.all) <- list(sort(unique(as.vector(mark1))), sort(unique(as.vector(mark2))), NULL)
+      dimnames(means.all) <- list(mark1.level, mark2.level, NULL)
       vars.all <- array(NA, c(length(mark1.level), length(mark2.level), ndraws))
-      dimnames(vars.all) <- list(sort(unique(as.vector(mark1))), sort(unique(as.vector(mark2))), NULL)
+      dimnames(vars.all) <- list(mark1.level, mark2.level, NULL)
       weight <- rep(0, ndraws) # weight for draws
       # loop thru draws
       for(i in 1:ndraws) {
         mark1.tmp <- mark1[,i] # data for current draw
         mark2.tmp <- mark2[,i]
         # fit a regression - this is used to calculate the weights
-        mark1.factor <- as.factor(mark1.tmp)
-        mark2.factor <- as.factor(mark2.tmp)
+        mark1.factor <- factor(mark1.tmp, mark1.level)
+        mark2.factor <- factor(mark2.tmp, mark2.level)
         lm.tmp <- lm(pheno~mark1.factor+mark2.factor+1)
         rss <- sum(lm.tmp$residuals^2)
         # compute the weight
@@ -637,14 +658,30 @@ effectplot.calmeanse <- function(pheno, mark1, mark2, ndraws, var.flag=c("pooled
       ses <- sqrt(meanvar+varmean)
     }
     else { # ndraws is 1
+      u <- sort(unique(mark1))
+      if(any(!(u %in% seq(along=geno1)))) {
+        newmark1 <- mark1
+        for(i in seq(along=u)) 
+          newmark1[mark1==u[i]] <- i
+        mark1 <- newmark1
+      }
+
+      u <- sort(unique(mark2))
+      if(any(!(u %in% seq(along=geno2)))) {
+        newmark2 <- mark2
+        for(i in seq(along=u)) 
+          newmark2[mark2==u[i]] <- i
+        mark2 <- newmark2
+      }
+      mark1 <- factor(mark1, seq(along=geno1))
+      mark2 <- factor(mark2, seq(along=geno2))
+
       means <- tapply(pheno, list(mark1, mark2), mean, na.rm = TRUE)
       if(var.flag=="group") { # use group variance
         ses <- tapply(pheno, list(mark1, mark2), function(a) sd(a, na.rm = TRUE)/sqrt(sum(!is.na(a))))
       }
       else {# use pooled variance
-        mark1.factor <- as.factor(mark1)
-        mark2.factor <- as.factor(mark2)
-        lm.tmp <- lm(pheno~mark1.factor+mark2.factor-1)
+        lm.tmp <- lm(pheno~mark1+mark2-1)
         rss <- sum(lm.tmp$residuals^2)
         ses <- tapply(mark1, list(mark1, mark2), function(a) sqrt(rss/nind/length(a)))
       }
