@@ -2,9 +2,9 @@
 #
 # qtlcart_io.R
 #
-# copyright (c) 2002-6, Brian S. Yandell
+# copyright (c) 2002-7, Brian S. Yandell
 #          [with some modifications by Karl W. Broman and Hao Wu]
-# last modified Nov, 2006
+# last modified Nov, 2007
 # first written Jun, 2002
 # Licensed under the GNU General Public License version 2 (June, 1991)
 #
@@ -250,8 +250,8 @@ function( cross, filestem="data")
   }
 
   # RIL data: convert genotypes to 1/3; later will be converted to 0/2
-  if(type=="riself" || type=="risib") {
-    for(i in 1:nchr) {
+  if(type=="RI1" || type=="RI2") {
+    for(i in 1:n.chr) {
       g <- cross$geno[[i]]$data
       g[!is.na(g) & g==2] <- 3
       cross$geno[[i]]$data <- g
@@ -261,8 +261,8 @@ function( cross, filestem="data")
   # write genotype and phenotype data
   file <- paste(filestem, ".cro", sep="")
   if( file.exists( file )) {
-    warning( paste( "previous file", file, "moved to *.mov" ))
-    file.rename( file, paste( file, "mov", sep = "." ))
+    warning( paste( "previous file", file, "moved to *.bak" ))
+    file.rename( file, paste( file, "bak", sep = "." ))
   }
   write("#  123456789 -filetype Rcross.out", file, append=FALSE)
 
@@ -312,8 +312,8 @@ function( cross, filestem="data")
   # make "prep" file with map information
   file <- paste(filestem, ".map", sep="")
   if( file.exists( file )) {
-    warning( paste( "previous file", file, "moved to *.mov" ))
-    file.rename( file, paste( file, "mov", sep = "." ))
+    warning( paste( "previous file", file, "moved to *.bak" ))
+    file.rename( file, paste( file, "bak", sep = "." ))
   }
   write("#  123456789 -filetype Rmap.out", file, append=FALSE)
 
@@ -352,15 +352,19 @@ function( cross, filestem="data")
   ncmap <- nchar( mapmat[1] )
   mapmat[ grep( "NA", mapmat ) ] <- paste( rep( " ", ncmap ), collapse = "" )
   tmp <- format( seq( n.chr ))
-  write( paste( "Marker    | ",
-               paste( tmp, collapse =
+  write( paste( "Marker    |  ",
+               paste( paste(" ", tmp, sep=""), collapse =
                      paste( rep( " ", max( 1, ncmap - nchar( tmp ))), collapse = "" ))),
         file, append=TRUE)
   write( "--------------------------------------", file, append=TRUE)
-  for( i in seq( nrow( mapmat )))
-    write( paste( "-l     ", i - 1, "|",
+  for( i in seq( nrow( mapmat ))) {
+    j <- 5-nchar(i-1)
+    if(j < 1) j <- 1
+    write( paste( "-l", paste(rep(" ", j), collapse=""),
+                 i - 1, "|",
                  paste( mapmat[i,], collapse = " " )),
           file, append=TRUE)
+  }
 
   write( "---------------------------------------", file, append=TRUE)
   write( paste( "-Number   |", paste( maplen, collapse = "  " )),
